import 'dart:collection';
import 'dart:io';

import 'package:collection/collection.dart' show IterableExtension;
import 'package:flutter/widgets.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/cubit/panda.state.dart';
import 'package:panda/models/public.dart';
import 'package:panda/repositories/panda.repository.dart';
import 'package:panda/utils/validator-config.utils.dart';
import 'package:rxdart/rxdart.dart';

class PandaCubit extends NiceBaseCubit<PandaState> {
  factory PandaCubit.of(BuildContext context) => BlocProvider.of<PandaCubit>(context);

  PandaRepository _repository;

  final LinkedHashMap<String, BehaviorSubject<LinkedHashMap<String, dynamic>?>> _subjects = LinkedHashMap.from({});
  final LinkedHashMap<String, BehaviorSubject<PandaFormValidation?>> _errorSubjects = LinkedHashMap.from({});

  final BehaviorSubject<PandaFormResultModel?> _onChange = BehaviorSubject<PandaFormResultModel?>();
  final BehaviorSubject<bool> _onValidChange = BehaviorSubject<bool>();

  Stream<PandaFormResultModel?> get onResultChange => _onChange.stream;

  Stream<bool> get onValidChange => _onValidChange.stream;

  PandaCubit(this._repository) : super(const PandaState.initialState()) {
    emit(state.copyWith(loading: true));
  }

  @override
  Future<void> close() {
    _onChange.close();
    _subjects.forEach((_, subject) => subject.close());
    _errorSubjects.forEach((_, subject) => subject.close());
    return super.close();
  }

  void load(PandaFormDescriptorModel? formDescriptor, PandaFormResultModel? formResult) {
    wrap(
      loading: false,
      callback: () {
        if (formDescriptor?.id == null) {
          throw "Form descriptor is missing";
        }

        formResult ??= PandaFormResultModel(
          id: null,
          formDescriptorId: formDescriptor!.id,
        );

        formResult!.value ??= LinkedHashMap<String, dynamic>.from({});

        emit(
          state.copyWith(
            loading: false,
            formDescriptor: formDescriptor,
            formResult: formResult,
            formValidation: PandaFormValidation(
              errors: [],
              values: LinkedHashMap<String, PandaFormValidation>(),
            ),
          ),
        );
      },
    );
  }

  void updateFormResult(PandaFormResultModel formResult) {
    _onChange.add(state.formResult);
    emit(state.copyWith(formResult: formResult.shallowClone()));
  }

  void updateFormValidation(PandaFormValidation formValidation) {
    _onValidChange.add(!PandaValidatorConfigUtils.hasError(state.formValidation));
    if (!isClosed) emit(state.copyWith(formValidation: formValidation.shallowClone()));
  }

  void submit() {
    emit(state.copyWith(submitted: true));
  }

  PandaViewConfigModel? getViewConfigByKeyTree(List<String?> keyTree) {
    if (state.formDescriptor == null || keyTree.isEmpty) {
      return null;
    }

    PandaViewConfigModel? viewConfig;
    for (String? nodeKey in keyTree) {
      // First iteration, viewConfig is null. If it is a group viewConfig, look in its children,
      // otherwise look in the appendables.
      if (viewConfig == null) {
        viewConfig = state.formDescriptor!.viewConfigs!.firstWhereOrNull((x) => x.keyName == nodeKey);
      } else if (viewConfig.type == PandaViewConfigType.Group) {
        viewConfig = (viewConfig.typeConfig as PandaGroupTypeConfigModel)
            .viewConfigs
            .firstWhereOrNull((x) => x.keyName == nodeKey);
      } else if (viewConfig.type == PandaViewConfigType.Table) {
        return (viewConfig.typeConfig as PandaTableTypeConfigModel)
            .colHeaders!
            .firstWhereOrNull((x) => x.viewConfig?.keyName == nodeKey)
            ?.viewConfig;
      } else {
        viewConfig = viewConfig.appendables?.firstWhereOrNull((x) => x.viewConfig?.keyName == nodeKey)?.viewConfig;
      }

      // ViewConfig is null, we cannot look deeper.
      if (viewConfig == null) {
        break;
      }
    }

    return viewConfig;
  }

  Stream<LinkedHashMap<String, dynamic>?> getValueObservableFromResult(List<String?> keyTree) {
    return getBehaviorSubjectFromResult(keyTree)!.stream;
  }

  // Maybe create and returns a new Subject for a keyTree, seeded with the form result node value at creation.
  // Adds a list method intercepting all writes to update the form result.
  BehaviorSubject<LinkedHashMap<String, dynamic>?>? getBehaviorSubjectFromResult(List<String?> keyTree) {
    final keyTreeAsString = keyTree.join(".");
    if (!_subjects.containsKey(keyTreeAsString)) {
      _subjects[keyTreeAsString] = BehaviorSubject<LinkedHashMap<String, dynamic>?>.seeded(_getFormResultNode(keyTree))
        ..skip(1).listen((LinkedHashMap<String, dynamic>? newNodeValue) {
          _setFormResultNode(keyTree, newNodeValue!);
        });
    }

    return _subjects[keyTreeAsString];
  }

  void deleteBehaviorSubjectFormResult(List<String?> keyTree) {
    final keyTreeAsString = keyTree.join(".");
    if (!_subjects.containsKey(keyTreeAsString)) return;

    _subjects.remove(keyTreeAsString)?.close();

    if (state.formResult == null) return;

    var immediateParent = state.formResult!.value;
    for (final subNodeKey in keyTree.sublist(0, keyTree.length - 1)) {
      if (immediateParent is! LinkedHashMap<String?, dynamic>) return;
      if (!immediateParent!.containsKey(subNodeKey)) return;
      immediateParent = immediateParent[subNodeKey];
    }

    immediateParent!.remove(keyTree.last);
  }

  // Finds and returns the node corresponding to the keyTree.
  LinkedHashMap<String, dynamic>? _getFormResultNode(List<String?> keyTree) {
    if (state.formResult == null) {
      return null;
    }

    LinkedHashMap<String, dynamic>? node = state.formResult!.value;

    // Key should lead to a Map<String, dynamic> or we create it. Otherwise dispatch error.
    for (String? subNodeKey in keyTree) {
      if (node is! LinkedHashMap<String?, dynamic>) {
        // Form result value structure is corrupted, avoid overriding possibly important value and dispatch error.
        throw "Unable to find node for keyTree: $keyTree";
      }

      if (!node!.containsKey(subNodeKey)) {
        node[subNodeKey!] = LinkedHashMap<String, dynamic>.from({});
      }

      node = node[subNodeKey!];
    }

    return node;
  }

  // Finds the node corresponding to the keyTree and merges with override the newNodeValue in the node.
  void _setFormResultNode(List<String?> keyTree, LinkedHashMap<String, dynamic> newNodeValue) {
    _getFormResultNode(keyTree)!.addAll(newNodeValue);
    if (state.formResult != null) updateFormResult(state.formResult!);
  }

  // Maybe creates then returns a new Subject for a keyTree, seeded with the form validation node value at creation.
  // Adds a list method intercepting all writes to update the form validation.
  BehaviorSubject<PandaFormValidation?>? getBehaviorSubjectFromErrors(List<String> keyTree) {
    final keyTreeAsString = keyTree.join(".");
    if (!_errorSubjects.containsKey(keyTreeAsString)) {
      _errorSubjects[keyTreeAsString] = BehaviorSubject<PandaFormValidation?>.seeded(
        PandaValidatorConfigUtils.findFormValidation(keyTree, state.formValidation),
      )..skip(1).listen((PandaFormValidation? newNodeFormValidation) {
          _setFormValidationNode(keyTree, newNodeFormValidation!);
        });
    }

    return _errorSubjects[keyTreeAsString];
  }

  // Finds the node corresponding to the keyTree and merges with override the newNodeValue in the node.
  void _setFormValidationNode(List<String> keyTree, PandaFormValidation newNodeFormValidation) {
    PandaFormValidation currentFormValidation =
        PandaValidatorConfigUtils.findFormValidation(keyTree, state.formValidation)!;
    currentFormValidation.errors = newNodeFormValidation.errors;
    currentFormValidation.values = newNodeFormValidation.values;
    if (state.formValidation != null) updateFormValidation(state.formValidation!);
  }

  Future<String> uploadFile(File file) {
    return _repository.uploadFile(file);
  }

  Future<String> getDownloadUrl(String fileId) {
    return _repository.getDownloadUrl(fileId);
  }
}
