import 'dart:async';
import 'dart:collection';
import 'dart:io';

import 'package:flutter/widgets.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:panda/cubit/panda.cubit.dart';
import 'package:panda/models/form-validation.model.dart';
import 'package:panda/models/validation-config/validation-error.model.dart';
import 'package:panda/models/view-config.model.dart';
import 'package:panda/utils/validator-config.utils.dart';
import 'package:panda/widgets/base-widget.dart';
import 'package:rxdart/rxdart.dart';

abstract class PandaBaseWidgetState<W extends PandaBaseWidget, ConfigType> extends State<W> {
  @protected
  late PandaCubit cubit;
  PandaViewConfigModel? viewConfig;

  // ignore: close_sinks
  BehaviorSubject<LinkedHashMap<String, dynamic>?>? node;

  // ignore: close_sinks
  BehaviorSubject<PandaFormValidation?>? errorNode;

  ConfigType? get typeConfig => viewConfig!.typeConfig as ConfigType?;

  StreamSubscription? validationSubscription;

  @override
  @mustCallSuper
  void didChangeDependencies() {
    super.didChangeDependencies();

    cubit = BlocProvider.of<PandaCubit>(context);
    viewConfig = cubit.getViewConfigByKeyTree(widget.keyTree);
    node = cubit.getBehaviorSubjectFromResult(widget.keyTree);
    errorNode = cubit.getBehaviorSubjectFromErrors(widget.keyTree);

    // VALIDATION - Each component instance listens on its model and each change in the model requires a new validation.
    validationSubscription ??= node!.startWith(node!.valueOrNull).listen(computeErrors);
  }

  @override
  void dispose() {
    validationSubscription?.cancel();
    clearErrors();
    super.dispose();
  }

  @protected
  void computeErrors(LinkedHashMap<String, dynamic>? model) {
    final List<PandaValidationError> errors = (viewConfig?.validationConfig?.validators ?? [])
        .map((validator) => PandaValidatorConfigUtils.validate(validator, model))
        .fold([], (prev, curr) => [...prev, ...curr]);

    errorNode!.sink.add(
      PandaFormValidation(
        errors: errors,
        values: errorNode!.value?.values ?? LinkedHashMap<String, PandaFormValidation>.from({}),
      ),
    );
  }

  @protected
  void clearErrors() {
    if (errorNode!.isClosed) return;
    errorNode!.sink.add(
      PandaFormValidation(
        errors: const [],
        values: errorNode!.value?.values ?? LinkedHashMap<String, PandaFormValidation>.from({}),
      ),
    );
  }

  Future<String?> uploadFile(File file) async {
    return cubit.uploadFile(file);
  }

  Future<String> getDownloadUrl(String fileId) {
    return cubit.getDownloadUrl(fileId);
  }
}
