import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/cubit/panda.cubit.dart';
import 'package:panda/models/view-config-types/group-type-config.model.dart';
import 'package:panda/models/view-config.model.dart';
import 'package:panda/widgets/form-view.widget.dart';
import 'package:preload_page_view/preload_page_view.dart';

class PandaSectionNavigator extends StatefulWidget {
  final int openAtPage;

  const PandaSectionNavigator(this.openAtPage, {Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _PandaSectionNavigator();
}

class _PandaSectionNavigator extends State<PandaSectionNavigator> {
  late final PreloadPageController _preloadPageController;

  PandaCubit get _bloc => PandaCubit.of(context);
  int _currentPage = 0;
  int _totalPage = 0;

  set page(int newPage) {
    setState(() {
      _currentPage = newPage;
      _preloadPageController.animateToPage(
        _currentPage,
        duration: const Duration(milliseconds: 400),
        curve: Curves.easeOut,
      );
    });
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    _preloadPageController = PreloadPageController(initialPage: widget.openAtPage);
    _currentPage = widget.openAtPage;
    _totalPage = _bloc.state.formDescriptor?.viewConfigs?.length ?? 0;
  }

  @override
  Widget build(BuildContext context) {
    if (_totalPage > 0) {
      return Dismissible(
        key: const Key("section-navigator"),
        onDismissed: (_) => Navigator.of(context).pop(),
        direction: DismissDirection.down,
        child: Scaffold(
          backgroundColor: Colors.transparent,
          body: _buildPages(),
        ),
      );
    }

    return SafeArea(
      child: Scaffold(
        body: Center(
          child: Text(
            NiceLocalizations.of(context).translate("panda.no_sections"),
          ),
        ),
      ),
    );
  }

  Widget _buildPages() {
    if (_totalPage > 0) {
      return SafeArea(
        child: Container(
          margin: const EdgeInsets.only(top: 8.0),
          decoration: const BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.only(
              topRight: Radius.circular(16),
              topLeft: Radius.circular(16),
            ),
          ),
          child: Column(
            children: <Widget>[
              const SizedBox(height: 12.0),
              Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: <Widget>[
                  Container(
                    width: 30,
                    height: 5,
                    decoration: BoxDecoration(
                      color: Colors.grey[300],
                      borderRadius: const BorderRadius.all(Radius.circular(12.0)),
                    ),
                  ),
                ],
              ),
              Expanded(
                child: PreloadPageView(
                  controller: _preloadPageController,
                  onPageChanged: (int index) {
                    page = index;
                  },
                  children: _bloc.state.formDescriptor!.viewConfigs!
                      .map((PandaViewConfigModel viewConfig) => _buildPage(viewConfig))
                      .toList(),
                ),
              ),
            ],
          ),
        ),
      );
    }

    return Center(
      child: Text(
        NiceLocalizations.of(context).translate("panda.no_sections"),
      ),
    );
  }

  Widget _buildPageTitle() {
    return Container(
      padding: const EdgeInsets.all(24.0),
      alignment: Alignment.centerLeft,
      child: Column(
        children: <Widget>[
          Row(
            children: <Widget>[
              CircleAvatar(
                backgroundColor: Theme.of(context).colorScheme.secondary.withOpacity(0.2),
                child: Text(
                  "${_currentPage + 1}",
                  style: TextStyle(
                    fontSize: 18.0,
                    fontWeight: FontWeight.w800,
                    color: Theme.of(context).colorScheme.secondary,
                  ),
                ),
              ),
              const SizedBox(width: 16.0),
              Expanded(
                child: Text(
                  _bloc.state.formDescriptor!.viewConfigs![_currentPage].title ??
                      NiceLocalizations.of(context).translate("panda.no_title"),
                  style: const TextStyle(fontSize: 26.0, fontWeight: FontWeight.w700, color: Colors.black87),
                  maxLines: 3,
                  overflow: TextOverflow.ellipsis,
                ),
              ),
            ],
          ),
          Container(
            height: 2.0,
            margin: const EdgeInsets.only(top: 16.0),
            child: LinearProgressIndicator(
              value: (_currentPage + 1) / _bloc.state.formDescriptor!.viewConfigs!.length,
              color: Theme.of(context).colorScheme.secondary,
              backgroundColor: Theme.of(context).colorScheme.secondary.withOpacity(0.2),
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildPage(PandaViewConfigModel groupViewConfig) {
    final typeConfig = groupViewConfig.typeConfig as PandaGroupTypeConfigModel;
    return Container(
      margin: const EdgeInsets.symmetric(vertical: 12.0),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: [
          _buildPageTitle(),
          Expanded(
            child: SingleChildScrollView(
              padding: const EdgeInsets.symmetric(horizontal: 24.0),
              child: Column(
                children: typeConfig.viewConfigs.map(
                  (PandaViewConfigModel viewConfig) {
                    return BlocProvider.value(
                      value: _bloc,
                      child: PandaFormViewWidget(
                        [
                          groupViewConfig.keyName,
                          viewConfig.keyName,
                        ],
                      ),
                    );
                  },
                ).toList(),
              ),
            ),
          ),
          Container(
            height: 72.0,
            color: Colors.white,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                if (_currentPage > 0)
                  IconButton(
                    onPressed: () => page = _currentPage - 1,
                    icon: const Icon(
                      Icons.chevron_left,
                      size: 14.0,
                    ),
                  ),
                if (_currentPage < _totalPage - 1)
                  OutlinedButton(
                    onPressed: _currentPage < _totalPage - 1 ? () => page = _currentPage + 1 : null,
                    child: Row(
                      children: [
                        Text(
                          NiceLocalizations.of(context).translate("panda.next_section"),
                          style: const TextStyle(
                            fontSize: 18.0,
                            fontWeight: FontWeight.w700,
                          ),
                        ),
                        const SizedBox(width: 12.0),
                        const Icon(
                          Icons.chevron_right,
                          size: 14.0,
                        ),
                      ],
                    ),
                  ),
                if (_currentPage == _totalPage - 1)
                  OutlinedButton(
                    onPressed: () => Navigator.of(context).pop(),
                    child: Row(
                      children: [
                        Text(
                          NiceLocalizations.of(context).translate("panda.finish_section"),
                          style: const TextStyle(
                            fontSize: 18.0,
                            fontWeight: FontWeight.w700,
                          ),
                        ),
                        const SizedBox(width: 12.0),
                        const Icon(
                          Icons.check,
                          size: 14.0,
                        ),
                      ],
                    ),
                  ),
              ],
            ),
          )
        ],
      ),
    );
  }
}
