import 'package:flutter/material.dart';
import 'package:nice_flutter_kit/nice_flutter_kit.dart';
import 'package:panda/extensions/iterable.extension.dart';
import 'package:panda/panda.dart';

class PandaTableWidget extends PandaBaseWidget {
  const PandaTableWidget(List<String> keyTree, {Key? key}) : super(keyTree, key: key);

  @override
  State<StatefulWidget> createState() => _PandaTableWidgetState();
}

class _PandaTableWidgetState extends PandaBaseWidgetState<PandaTableWidget, PandaTableTypeConfigModel> {
  int _rowCount = 1;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();

    if (typeConfig == null) return;
    _rowCount = typeConfig!.rowHeaders?.length ?? 1;
  }

  @override
  Widget build(BuildContext context) {
    if (viewConfig == null) return const SizedBox.shrink();

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        if (viewConfig!.title?.isNotEmpty ?? false)
          Text(
            viewConfig!.title!,
            style: const TextStyle(fontWeight: FontWeight.bold),
          ),
        Padding(
          padding: const EdgeInsets.symmetric(vertical: 12.0),
          child: _buildTable(),
        ),
        if (typeConfig!.rowHeaders == null) _buildAddRowButton(),
      ],
    );
  }

  Widget _buildTable() {
    return StreamBuilder<Map<String, dynamic>?>(
      initialData: node!.valueOrNull ?? const {"value": null},
      stream: node!.stream.distinct(),
      builder: (context, snapshot) {
        return Table(
          defaultVerticalAlignment: TableCellVerticalAlignment.top,
          columnWidths: {
            if (typeConfig!.rowHeaders == null) typeConfig!.colHeaders!.length: const FixedColumnWidth(64),
          },
          children: [
            _buildTableHeader(),
            for (int rowIndex = 0; rowIndex < _rowCount; rowIndex++) _buildTableRow(rowIndex),
          ],
        );
      },
    );
  }

  TableRow _buildTableHeader() {
    return TableRow(
      children: [
        // Col for row headers
        if (typeConfig?.rowHeaders != null) const SizedBox.shrink(),
        for (final colHeader in typeConfig!.colHeaders!)
          Padding(
            padding: const EdgeInsets.all(12),
            child: Text(colHeader.title ?? ""),
          ),
        // Col for remove buttons
        if (typeConfig?.rowHeaders == null) const SizedBox.shrink(),
      ],
    );
  }

  TableRow _buildTableRow(int rowIndex) {
    return TableRow(
      children: [
        if (typeConfig!.rowHeaders != null)
          Padding(
            padding: const EdgeInsets.only(top: 12),
            child: Text(
              typeConfig!.rowHeaders!.elementAtOrNull(rowIndex) ?? "",
            ),
          ),
        for (int colIndex = 0; colIndex < typeConfig!.colHeaders!.length; colIndex++)
          Padding(
            padding: const EdgeInsets.symmetric(horizontal: 8),
            child: PandaFormViewWidget(
              key: ValueKey("$rowIndex $colIndex $_rowCount"),
              [...widget.keyTree, typeConfig!.colHeaders![colIndex].viewConfig!.keyName, "$rowIndex"],
            ),
          ),
        if (typeConfig!.rowHeaders == null) _buildDeleteRowButton(rowIndex),
      ],
    );
  }

  Widget _buildAddRowButton() {
    return TextButton(
      onPressed: _addRow,
      child: Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          Text(
            NiceLocalizations.of(context).translate("panda.add_row"),
          ),
          const SizedBox(width: 8),
          const Icon(Icons.add),
        ],
      ),
    );
  }

  Widget _buildDeleteRowButton(int rowIndex) {
    if (_rowCount <= 1) return const SizedBox.shrink();

    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 12),
      child: IconButton(
        onPressed: () => _deleteRow(rowIndex),
        color: Theme.of(context).primaryColor,
        icon: const Icon(Icons.clear),
      ),
    );
  }

  void _addRow() {
    setState(() {
      _rowCount++;
    });
  }

  void _deleteRow(int rowIndex) {
    for (final colHeader in typeConfig!.colHeaders!) {
      final colKeyName = colHeader.viewConfig!.keyName;

      for (int i = rowIndex; i < _rowCount; i++) {
        if (i == _rowCount - 1) {
          cubit.deleteBehaviorSubjectFormResult([...widget.keyTree, colKeyName, "$i"]);
        } else {
          // ignore: close_sinks
          final nextNode = cubit.getBehaviorSubjectFromResult([...widget.keyTree, colKeyName, "${i + 1}"]);

          // ignore: close_sinks
          final currentNode = cubit.getBehaviorSubjectFromResult([...widget.keyTree, colKeyName, "$i"]);
          currentNode?.sink.add(nextNode!.value);
        }
      }
    }

    setState(() {
      _rowCount--;
    });
  }
}
